@@ -42,12 +42,13 @@ static std::string const CERT_PARAM_LENGTH_CHECK_DEFAULT("127");
 void
 galera::Certification::register_params(gu::Config& cnf)
 {
-    cnf.add(CERT_PARAM_LOG_CONFLICTS, CERT_PARAM_LOG_CONFLICTS_DEFAULT);
-    cnf.add(CERT_PARAM_OPTIMISTIC_PA, CERT_PARAM_OPTIMISTIC_PA_DEFAULT);
+    const int flags(gu::Config::Flag::type_bool);
+    cnf.add(CERT_PARAM_LOG_CONFLICTS, CERT_PARAM_LOG_CONFLICTS_DEFAULT, flags);
+    cnf.add(CERT_PARAM_OPTIMISTIC_PA, CERT_PARAM_OPTIMISTIC_PA_DEFAULT, flags);
     /* The defaults below are deliberately not reflected in conf: people
      * should not know about these dangerous setting unless they read RTFM. */
-    cnf.add(CERT_PARAM_MAX_LENGTH);
-    cnf.add(CERT_PARAM_LENGTH_CHECK);
+    cnf.add(CERT_PARAM_MAX_LENGTH, gu::Config::Flag::hidden);
+    cnf.add(CERT_PARAM_LENGTH_CHECK, gu::Config::Flag::hidden);
 }
 
 /* a function to get around unset defaults in ctor initialization list */
@@ -187,10 +188,7 @@ check_against(const galera::KeyEntryNG*   const found,
             }
             /* fall through */
         case DEPENDENCY:
-            if (conflict)
-                depends_seqno = WSREP_SEQNO_UNDEFINED;
-            else
-                depends_seqno = std::max(ref_trx->global_seqno(), depends_seqno);
+            depends_seqno = std::max(ref_trx->global_seqno(), depends_seqno);
             /* fall through */
         case NOTHING:;
         }
@@ -206,6 +204,7 @@ certify_and_depend_v3to5(const galera::KeyEntryNG*   const found,
                          galera::TrxHandleSlave*     const trx,
                          bool                        const log_conflict)
 {
+    bool ret(false);
     wsrep_seqno_t depends_seqno(trx->depends_seqno());
     wsrep_key_type_t const key_type(key.wsrep_type(trx->version()));
 
@@ -237,14 +236,13 @@ certify_and_depend_v3to5(const galera::KeyEntryNG*   const found,
           check_against<WSREP_KEY_SHARED>
           (found, key, key_type, trx, log_conflict, depends_seqno))))
     {
-        return true;
-    }
-    else
-    {
-        if (depends_seqno > trx->depends_seqno())
-            trx->set_depends_seqno(depends_seqno);
-        return false;
+        ret = true;
     }
+
+    if (depends_seqno > trx->depends_seqno())
+        trx->set_depends_seqno(depends_seqno);
+
+    return ret;
 }
 
 /* returns true on collision, false otherwise */
@@ -378,6 +376,7 @@ galera::Certification::do_test_v3to5(TrxHandleSlave* trx, bool store_keys)
 
         if (certify_v3to5(cert_index_ng_, key, trx, store_keys, log_conflicts_))
         {
+            trx->set_depends_seqno(std::max(trx->depends_seqno(), last_pa_unsafe_));
             goto cert_fail;
         }
     }
@@ -495,8 +494,6 @@ galera::Certification::do_test(const TrxHandleSlavePtr& trx, bool store_keys)
                        << version_ << " not implemented";
     }
 
-    assert(TEST_FAILED == res || trx->depends_seqno() >= 0);
-
     if (store_keys == true && res == TEST_OK)
     {
         ++trx_count_;
@@ -990,6 +987,20 @@ galera::Certification::~Certification()
     for_each(trx_map_.begin(), trx_map_.end(), PurgeAndDiscard(*this));
     trx_map_.clear();
     nbo_map_.clear();
+    std::for_each(nbo_index_.begin(), nbo_index_.end(),
+                  [](CertIndexNBO::value_type key_entry)
+                  {
+                      for (int i(0); i <= KeySet::Key::TYPE_MAX; ++i)
+                      {
+                          wsrep_key_type_t key_type(static_cast<wsrep_key_type_t>(i));
+                          const TrxHandleSlave* ts(key_entry->ref_trx(key_type));
+                          if (ts)
+                          {
+                              key_entry->unref(key_type, ts);
+                          }
+                      }
+                      delete key_entry;
+                  });
     if (service_thd_)
     {
         service_thd_->release_seqno(position_);
@@ -1127,8 +1138,6 @@ galera::Certification::test(const TrxHandleSlavePtr& trx, bool store_keys)
         (trx->preordered() ?
          do_test_preordered(trx.get()) : do_test(trx, store_keys));
 
-    assert(TEST_FAILED == ret || trx->depends_seqno() >= 0);
-
     if (gu_unlikely(ret != TEST_OK)) { trx->mark_dummy(); }
 
     return ret;
