@@ -60,6 +60,11 @@ galera::ReplicatorSMM::InitConfig::InitConfig(gu::Config&       conf,
             conf.add(i->first, i->second);
     }
 
+    conf.set_flags(Param::max_write_set_size, gu::Config::Flag::type_integer);
+    conf.set_flags(Param::base_dir, gu::Config::Flag::read_only);
+    conf.set_flags(Param::base_port, gu::Config::Flag::read_only |
+                   gu::Config::Flag::type_integer);
+
     // what is would be a better protection?
     int const pv(gu::from_string<int>(conf.get(Param::proto_max)));
     if (pv > MAX_PROTO_VER)
@@ -69,8 +74,7 @@ galera::ReplicatorSMM::InitConfig::InitConfig(gu::Config&       conf,
         conf.add(Param::proto_max, gu::to_string(MAX_PROTO_VER));
     }
 
-    conf.add(COMMON_BASE_HOST_KEY);
-    conf.add(COMMON_BASE_PORT_KEY);
+    conf.add(COMMON_BASE_HOST_KEY, gu::Config::Flag::read_only);
 
     if (node_address && strlen(node_address) > 0)
     {
@@ -197,7 +201,15 @@ galera::ReplicatorSMM::param_set (const std::string& key,
 {
     try
     {
+#ifdef GALERA_HAVE_SSL
+        // Param `socket.ssl_reload` is used as trigger. We should
+        // skip checking if value is changed.
+        if (key != gu::conf::ssl_reload) {
+#endif
         if (config_.get(key) == value) return;
+#ifdef GALERA_HAVE_SSL
+        }
+#endif
     }
     catch (gu::NotSet&) {}
 
@@ -236,6 +248,15 @@ galera::ReplicatorSMM::param_set (const std::string& key,
             found = true;
         }
         catch (gu::NotFound&) {}
+
+#ifdef GALERA_HAVE_SSL
+        try
+        {
+            gu::ssl_param_set (key, value, config_);
+            found = true;
+        }
+        catch (gu::NotFound&) {}
+#endif
     }
 
     if (!found) throw gu::NotFound();
